@@ -27,15 +27,15 @@
 #include "nameclash.h"
 #include "file_name.h"
 
-void label_name(doscp_t *cp, const char *filename, int verbose,
-		int *mangled, dos_name_t *ans)
+static void _label_name(doscp_t *cp, const char *filename, int verbose,
+			int *mangled, dos_name_t *ans, int preserve_case)
 {
 	int len;
 	int i;
 	int have_lower, have_upper;
 	wchar_t wbuffer[12];
 
-	memset(ans, ' ', sizeof(ans)-1);
+	memset(ans, ' ', sizeof(*ans)-1);
 	ans->sentinel = '\0';
 	len = native_to_wchar(filename, wbuffer, 11, 0, 0);
 	if(len > 11){
@@ -50,7 +50,8 @@ void label_name(doscp_t *cp, const char *filename, int verbose,
 			have_lower = 1;
 		if(isupper(wbuffer[i]))
 			have_upper = 1;
-		wbuffer[i] = towupper(wbuffer[i]);
+		if(!preserve_case)
+			wbuffer[i] = towupper(wbuffer[i]);
 		if(
 #ifdef HAVE_WCHAR_H
 		   wcschr(L"^+=/[]:,?*\\<>|\".", wbuffer[i])
@@ -67,6 +68,18 @@ void label_name(doscp_t *cp, const char *filename, int verbose,
 	wchar_to_dos(cp, wbuffer, ans->base, len, mangled);
 }
 
+void label_name_uc(doscp_t *cp, const char *filename, int verbose,
+		   int *mangled, dos_name_t *ans)
+{
+	_label_name(cp, filename, verbose, mangled, ans, 0);
+}
+
+void label_name_pc(doscp_t *cp, const char *filename, int verbose,
+		   int *mangled, dos_name_t *ans)
+{
+	_label_name(cp, filename, verbose, mangled, ans, 1);
+}
+
 int labelit(struct dos_name_t *dosname,
 	    char *longname,
 	    void *arg0,
@@ -93,7 +106,7 @@ static void usage(int ret)
 void mlabel(int argc, char **argv, int type)
 {
 
-	char *newLabel;
+	const char *newLabel="";
 	int verbose, clear, interactive, show;
 	direntry_t entry;
 	int result=0;
@@ -115,9 +128,10 @@ void mlabel(int argc, char **argv, int type)
 	struct label_blk_t *labelBlock;
 	int isRo=0;
 	int *isRop=NULL;
+	char drive;
 
 	init_clash_handling(&ch);
-	ch.name_converter = label_name;
+	ch.name_converter = label_name_uc;
 	ch.ignore_entry = -2;
 
 	verbose = 0;
@@ -129,7 +143,7 @@ void mlabel(int argc, char **argv, int type)
 	while ((c = getopt(argc, argv, "i:vcsnN:h")) != EOF) {
 		switch (c) {
 			case 'i':
-				set_cmd_line_image(optarg, 0);
+				set_cmd_line_image(optarg);
 				break;
 			case 'v':
 				verbose = 1;
@@ -147,7 +161,7 @@ void mlabel(int argc, char **argv, int type)
 				break;
 			case 'N':
 				set_serial = SER_SET;
-				serial = strtol(optarg, &eptr, 16);
+				serial = strtoul(optarg, &eptr, 16);
 				if(*eptr) {
 					fprintf(stderr,
 						"%s not a valid serial number\n",
@@ -162,11 +176,18 @@ void mlabel(int argc, char **argv, int type)
 			}
 	}
 
-	if (argc - optind != 1 || !argv[optind][0] || argv[optind][1] != ':')
+	if (argc - optind > 1)
 		usage(1);
+	if(argc - optind == 1) {
+	    if(!argv[optind][0] || argv[optind][1] != ':')
+		usage(1);
+	    drive = toupper(argv[argc -1][0]);
+	    newLabel = argv[optind]+2;
+	} else {
+	    drive = get_default_drive();
+	}
 
 	init_mp(&mp);
-	newLabel = argv[optind]+2;
 	if(strlen(newLabel) > VBUFSIZE) {
 		fprintf(stderr, "Label too long\n");
 		FREE(&RootDir);
@@ -178,7 +199,13 @@ void mlabel(int argc, char **argv, int type)
 	if(!clear && !newLabel[0]) {
 		isRop = &isRo;
 	}
-	RootDir = open_root_dir(argv[optind][0], isRop ? 0 : O_RDWR, isRop);
+	if(clear && newLabel[0]) {
+		/* Clear and new label specified both */
+		fprintf(stderr, "Both clear and new label specified\n");
+		FREE(&RootDir);
+		exit(1);
+	}		
+	RootDir = open_root_dir(drive, isRop ? 0 : O_RDWR, isRop);
 	if(isRo) {
 		show = 1;
 		interactive = 0;
@@ -209,14 +236,26 @@ void mlabel(int argc, char **argv, int type)
 
 	/* ask for new label */
 	if(interactive){
+		saved_sig_state ss; 
 		newLabel = longname;
+		allow_interrupts(&ss);
 		fprintf(stderr,"Enter the new volume label : ");
-		if(fgets(newLabel, VBUFSIZE, stdin) == NULL) {
-			newLabel[0] = '\0';
+		if(fgets(longname, VBUFSIZE, stdin) == NULL) {
 			fprintf(stderr, "\n");
+			if(errno == EINTR) {
+				FREE(&RootDir);
+				exit(1);
+			}
+			longname[0] = '\0';
 		}
-		if(newLabel[0])
-			newLabel[strlen(newLabel)-1] = '\0';
+		if(longname[0])
+			longname[strlen(newLabel)-1] = '\0';
+	}
+
+	if(strlen(newLabel) > 11) {
+		fprintf(stderr,"New label too long\n");
+		FREE(&RootDir);
+		exit(1);
 	}
 
 	if((!show || newLabel[0]) && !isNotFound(&entry)){
@@ -258,7 +297,7 @@ void mlabel(int argc, char **argv, int type)
 		else
 			shrtLabel = newLabel;
 		cp = GET_DOSCONVERT(Fs);
-		label_name(cp, shrtLabel, verbose, &mangled, &dosname);
+		label_name_pc(cp, shrtLabel, verbose, &mangled, &dosname);
 
 		if(have_boot && boot.boot.descr >= 0xf0 &&
 		   labelBlock->dos4 == 0x29) {
@@ -278,6 +317,13 @@ void mlabel(int argc, char **argv, int type)
 
 	if(need_write_boot) {
 		force_write(Fs, (char *)&boot, 0, sizeof(boot));
+		/* If this is fat 32, write backup boot sector too */
+		if(!WORD_S(fatlen)) {
+			int backupBoot = WORD_S(ext.fat32.backupBoot);
+			force_write(Fs, (char *)&boot, 
+				    backupBoot * WORD_S(secsiz),
+				    sizeof(boot));
+		}
 	}
 
 	FREE(&RootDir);
